#*****************************************************************************
# Hands-On exercise: startR
# An-Chi - 30/05/2022
#*****************************************************************************

# 1. Define data
# The first step of the analysis is use Start() to identify the data needed.
# Hints:
# (1) The data paths use two '$' as the wildcard, and it can be given any names you like.
# (2) The variable we want to use is 'tas'.
# (3) The required time (sdate) period is November 1981 to 2016.
# (4) Read global data (i.e., full spatial grids.)
# (5) Take only the first time step.
# (6) To make the data smaller, we only take the first two ensemble members from the experimental data.
# (7) Because we are going to use the whole startR workflow, we only need to create a pointer to the data repository rather than retrieve it to the workstation.

library(startR)

# exp data
  # Use this one if on workstation or nord3
  repos_exp <- paste0('/esarchive/exp/ecmwf/system4_m1/monthly_mean/',
                      '$var$_f6h/$var$_$sdate$.nc')

  
  sdates_exp <- sapply(1981:2016, function(x) paste0(x, '1101'))

  exp <- Start(dat = repos_exp,
               var = 'tas',
               sdate = sdates_exp,
               time = 'first',
               ensemble = indices(1:2),
               latitude = 'all',
               longitude = 'all',
               synonims = list(longitude = c('lon', 'longitude'),
                               latitude = c('lat', 'latitude')), # what are synonims ??
               return_vars = list(latitude = NULL, longitude = NULL, time = 'sdate'),
               retrieve = FALSE)

# obs data
  repos_obs <- paste0('/esarchive/recon/ecmwf/erainterim/monthly_mean/$var$_f6h/$var$_$sdate$.nc') # Different path from documentation
  sdates_obs <- sapply(1981:2016, function(x) paste0(x, '11'))

  obs <- Start(dat = repos_obs,
               var = 'tas',
               sdate = sdates_obs,
               time = indices(1),
               latitude = 'all',
               longitude = 'all',
               synonims = list(longitude = c('lon', 'longitude'),
                               latitude = c('lat', 'latitude')),
               return_vars = list(latitude = NULL, longitude = NULL, time = 'sdate'),
               retrieve = FALSE)

# Question:

# What are the dimensions of these two data?
# exp: time, ensemble, latitude, longitude

# What is the size of these two data?
# > attr(exp, 'Dimension')
#       dat       var     sdate      time  ensemble  latitude longitude 
#         1         1        36         1         2       256       512 

# > attr(obs, 'Dimension')
#       dat       var     sdate      time  latitude longitude 
#         1         1        36         1       256       512 

# 2. Define operation and workflow

# It is recommended to define the function and write Step() together, because the latter one helps you clarify the input and output dimensions of the function.
# In the self-defined function, we want to use the two functions, 'SpecsVerification::EnsCrps' and 's2dv:::.RMSSS', to calculate the skill score.
# You can check the first function by typing ?SpecsVerification::EnsCrps, and the second function on s2dv GitLab.

# EnsCrps: Calculate the ensemble-adjusted Continuous Ranked Probability Score (CRPS)

#'Compute the root mean square error skill score (RMSSS) between an array of 
#'forecast 'exp' and an array of observation 'obs'. The two arrays should 
#'have the same dimensions except along memb_dim, where the length can be 
#'different, with the number of experiments/models (nexp) and the number of 
#'observational datasets (nobs).

# Hint:
# (1) The self-defined function is for exp and obs data. Therefore, target_dims and output_dims in Step() should be a list with two vector elements.
# (2) Check the functions 'EnsCrps' and '.RMSSS'. What are the minimum required dimensions of inputs of each function? These dimensions should be the 'target_dims' in Step().
# (3) To return the two skill scores ('crps' and 'rmsss') together, put them in a list with two elements.
# (4) What are the dimensions of outputs? These dimensions should be the 'output_dims' in Step().
# (5) The first input of AddStep() should also be a list containing exp and obs.



?SpecsVerification::EnsCrps
# self-defined function
 
func <- function(exp, obs) {
  # CRPS
  crps <- mean(SpecsVerification::EnsCrps(exp, obs, R.new = Inf), na.rm = TRUE) # Time_dim

  # RMSSS
  # obs only has [sdate] now. Add one more dim for it.
  obs <- s2dv::InsertDim(obs, posdim = 2, lendim = 1, name = 'ensemble')
  rmsss <- mean(s2dv:::.RMSSS(exp, obs, time_dim = 'sdate', dat_dim = 'ensemble', pval = FALSE)$rmsss) # input: Nexp * Nobs (mem_dim can be different)

  return(list(crps = crps, rmsss = rmsss))
}
step <- Step(func, target_dims = list(c('sdate','ensemble'),c('sdate')), # Why 'sdate' dim appears twice??
                   output_dims = list(crps = NULL, rmsss = NULL)) # What does it mean NULL dim ??
wf <- AddStep(list(exp,obs), step)

# Question:

# Which dimensions are used in operation? Which dimensions are free?
# Can we use more dimensions as target_dims? What are the disadvantages?

# 3. Execute locally

# To avoid potential technical problems in the connection and configuration,
# we choose to run the execution locally.
# Noted that it is recommended to submit jobs to HPCs if the data size is big or
# the operation is heavy.
# Hint:
# (1) Use the free dimensions (i.e., those are not 'target_dims' in Step()) to chunk.
# (2) It is safe to divide the data into pieces of which the size each is 1/2-1/3 of RAM. In this case, the data size is only around 100Mb, so you can play with it without
# worrying about crashing the workstation. --> How to devide data into pieces??
# (3) You will see some error messages like "Error in R_nc4_open: No such file or directory",
# which are ignorable.

res <- Compute(wf$crps, chunks = list(latitude = 2, longitude = 2))

# 4. Check the results
# Check the list structure using function str(). What are the items in the list?
str(res)
# List of 2
#  $ crps : num [1, 1, 1, 1:256, 1:512] 0.879 0.97 1.029 1.04 0.991 ...
#  $ rmsss: num [1, 1, 1, 1:256, 1:512] 0.991 0.991 0.99 0.99 0.99 ...
#  - attr(*, "startR_compute_profiling")=List of 14
#   ..$ nchunks          : num 4
#   ..$ concurrent_chunks: num 1
#   ..$ cores_per_job    : logi NA
#   ..$ threads_load     : num 1
#   ..$ threads_compute  : num 1
#   ..$ bychunks_setup   : num 0.000577
#   ..$ transfer         : num 0
#   ..$ queue            : num 0
#   ..$ job_setup        : num 0
#   ..$ load             : num [1:4] 26.7 23 25.7 25.7
#   ..$ compute          : num [1:4] 8.89 9.12 9.13 9.14
#   ..$ transfer_back    : num 0
#   ..$ merge            : num 0.391
#   ..$ total            : num 138

# What do the dimensions of each item look like?
      # dat       var      time  latitude longitude 
      #   1         1         1       256       512 

# Plot the map
# Use s2dv::PlotEquiMap or other visualization tools to plot the map.

lat <- as.vector(attr(exp, 'Variables')$common$latitude) # What is common variable??
lon <- as.vector(attr(exp, 'Variables')$common$longitude)
 # Set the color bar 
brks_crps <- seq(min(res$crps, na.rm = TRUE), max(res$crps, na.rm = TRUE), length.out = 11)
brks_rmsss <- seq(min(res$rmsss, na.rm = TRUE), max(res$rmsss, na.rm = TRUE), length.out = 11)

library(s2dv)

PlotEquiMap(res$crps, lon, lat,color_fun = clim.palette('yellowred'), brks = brks_crps,
              filled.continents = FALSE, triangle_ends = c(TRUE, TRUE),
              toptitle = 'ECMWF monthly mean tas CRPS 2012-2016', title_scale = 0.6)

PlotEquiMap(res$rmsss, lon, lat, color_fun = clim.palette('yellowred'), brks = brks_rmsss,
              filled.continents = FALSE, triangle_ends = c(TRUE, TRUE),
              toptitle = 'ECMWF monthly mean tas RMSSS 2012-2016', title_scale = 0.6)
