########################################################################
# This script is used to calculate the mean of the observations
# April 2023
########################################################################

# Step (1): Load the libraries
library(CSTools)
library(lubridate)
library(s2dv)
library(ClimProjDiags)

# Step (2): Create daily data
obs <- array(1:90, dim = c(day = 90))

# Step (3): Create the dates
dates <- seq(ymd('2023-04-01'), ymd('2023-04-01') + days(89), by = 'day')

# Step (4): Add dimension month to data
indices_month <- month(dates)
data_month <- SplitDim(obs, split_dim = 'day', indices = indices_month, 
                       new_dim_name = 'month')

# Step (5): Compute the mean
data_montly_mean <- MeanDims(data_month, dims = 'day', na.rm = TRUE)

########################################################################
# Example 2: Compute only the daily average

# Step (1): Create hourly data
data_hourly <- array(1:200, dim = c(hours = 200))

# Step (2): Create the dates
dates <- seq(ymd_hms("2023-04-25 03:00:00"), ymd_hms("2023-04-25 03:00:00") + hours(199), by = 'hour')

#---------------------------------------------------
hourly_dataframe <- data.frame(dates = dates, data = data_hourly)
hourly_dataframe$days <- day(dates)
hourly_dataframe$months <- month(dates)
hourly_dataframe$hours <- hour(dates)

library(dplyr)
# Filter night-time data (8 PM to 5 AM)
nighttime_data <- hourly_dataframe %>% 
  filter(hours >= 17 | hours <= 4)

# Compute monthly mean
monthly_mean <- nighttime_data %>%
  group_by(months) %>%
  summarize(monthly_mean = mean(data, na.rm = TRUE))

# > monthly_mean
# # A tibble: 2 × 2
#   months monthly_mean
#    <dbl>        <dbl>
# 1      4         73.6
# 2      5        171. 

#---------------------------------------------------
# Step (1): Create hourly data
data_hourly <- array(1:200, dim = c(time = 200))

# Step (2): Create the dates
dates <- seq(ymd_hms("2023-04-25 03:00:00"), ymd_hms("2023-04-25 03:00:00") + hours(199), by = 'hour')


# Step (3): Add dimension hour

indices_hour <- hour(dates)

data_hour <- SplitDim(data_hourly, split_dim = 'time', indices = indices_hour, 
                     new_dim_name = 'hour')

dim(data_hour)


# Step (3): Add dimension day to data
indices_day <- unique(day(dates))

data_day <- SplitDim(data_hour, split_dim = 'time', indices = indices_day, 
                     new_dim_name = 'day')

# Step (4): Add dimension month

indices_month <- month(unique(date(dates)))
data_month <- SplitDim(data_day, split_dim = 'day', indices = indices_month, 
                       new_dim_name = 'month')

# Step (5): Subset over night hours
data_night <- Subset(data_month, along = 'hour', indices = c(1:5, 18:24))

# Step (6): Compute the mean
data_night_mean <- MeanDims(data_night, dims = c('hour', 'day'), na.rm = TRUE)

########################################################################



#---------------------------------------------------
# Example with arrays:
#---------------------------------------------------

# Step (0): Load the libraries
library(CSTools)
library(lubridate)
library(s2dv)
library(ClimProjDiags)

# Step (1): Create hourly data
data_hourly <- array(1:200, dim = c(time = 200))

# Step (2): Create the dates

ini_date <- ymd_hms("2023-04-25 03:00:00") 
end_date <- ymd_hms("2023-04-25 03:00:00") + hours(199)

indices_hour <- 0:23

hour_fill_first <- rep(NA, hour(ini_date))

hour_fill_last <- rep(NA, 24 - hour(end_date)-1)

data_hourly_complete <- c(hour_fill_first, as.vector(data_hourly), hour_fill_last)
dates_complete <- seq(floor_date(ini_date, unit = 'day'), 
                      floor_date(end_date, unit = 'day') + days(1) - hours(1), by = 'hour')

# Step (1): add hour index
dim(data_hourly_complete) <- c(time = length(data_hourly_complete))

# Step (2): Inster dim
indices_hour <- rep(1:24, length(dates_complete)/24)
data_hour <- SplitDim(data_hourly_complete, split_dim = 'time', 
                      indices = indices_hour, 
                      new_dim_name = 'hour')

# Step (3): Add dimension month to data
dates <- unique(date(dates_complete))
indices_month <- month(dates)
data_month <- SplitDim(data_hour, split_dim = 'time', indices = indices_month, 
                       new_dim_name = 'month')

dim(data_month)

# Step (4): Subset night
data_subset <- Subset(data_month, along = 'hour', indices = c(1:5, 18:24))

# Step (5): Compute the mean
data_mean <- MeanDims(data_subset, dims = c('hour', 'time'), na.rm = TRUE)

# > data_mean
# [1]  73.6087 170.8966